@@ -1,3 +1,3 @@
 from .PASTE import pairwise_align, center_align
-from .helper import kl_divergence, kl_divergence_backend, intersect, match_spots_using_spatial_heuristic, filter_for_common_genes
+from .helper import match_spots_using_spatial_heuristic, filter_for_common_genes, apply_trsf
 from .visualization import plot_slice, stack_slices_pairwise, stack_slices_center
\ No newline at end of file
